

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

import java.util.*;


/*
 * This class extends JPanel
 *
 * we maintain a linked list collection of sprite (image) objects
 *
 * when anybody asks this JPanel to draw itself by calling
 * repaint(), the Java Swing library will call paintComponent()
 * which is where we draw the visible images from our collection
 */
class GamePanel extends JPanel
{
   private LinkedList<Sprite> sprites;


   // the ImageObserver provides a way for us to know
   // when the animated sprite (the explosion) is finished
   // exploding
   private ImageObserver imageobserver = null;


   // constuctor
   public GamePanel(ImageObserver imageobserver)
   {
      sprites = new LinkedList<Sprite>();
      this.imageobserver = imageobserver;
   }


   // add a sprite
   public void registerSprite(Sprite sprite)
   {
      sprites.add(sprite);
   }


   // draw all of our sprites
   public void paintComponent(Graphics g)
   {
      super.paintComponent(g);
      Graphics2D g2 = (Graphics2D)g;

      for(Sprite s : sprites)
         if(s.isVisible())
            g2.drawImage(s.getImage(), s.getX(), s.getY(), imageobserver);
   }
}





